# -*- coding: utf-8 -*-

import os
import shutil
import subprocess

from .settings import (WELUT_REMOVED_EXTENSIONS, WELUT_IMAGES_EXTENSION)


class FileConverter(object):

    def __init__(self):
        pass

    def get_file_extension(self, value):
        """ return .pdf, .epub, .mobi """
        return '%s' % os.path.splitext(value)[-1]

    def get_final_filename(self, path, ext='.pdf'):
        """ rename filename from .epub, .mobi to specific file, default: .pdf """
        filename = path.split('/')[-1]
        file_extension = self.get_file_extension(filename)
        return filename.replace(file_extension, ext)

    def convert(self, _file):
        # eg: '/path/to/env-welut/welut/welut_demo/media/ebooks/2017/11/13/nama-file.epub'
        epub_file_full_path = _file.path

        # eg: ('/path/to/env-welut/welut/welut_demo/media/ebooks/2017/11/13/', 'nama-file.epub')
        splited_path = os.path.split(epub_file_full_path)
        current_path, origin_filename = splited_path[0], splited_path[-1]

        pdf_file = self.get_final_filename(epub_file_full_path, '.pdf')
        pdf_file_full_path = os.path.join(current_path, pdf_file)

        try:
            subprocess.call(['ebook-convert', epub_file_full_path, pdf_file_full_path])
            subprocess.call(['pdftocairo', '-png', pdf_file_full_path, current_path])

            # try to move all .png files from previous path to next path
            # ('/path/to/env-welut/welut/welut_demo/media/ebooks/2017/11', '13')
            prev_path_splited = os.path.split(current_path)
            prev_path, next_path = prev_path_splited[0], prev_path_splited[-1]

            # move all files to new destination path
            list_filenames = []
            for fname in os.listdir(prev_path):
                # 'nama-file'
                next_path_by_file = os.path.basename(os.path.splitext(epub_file_full_path)[0])

                # '/path/to/env-welut/welut/welut_demo/media/ebooks/2017/11/nama-file/13-01.png'
                next_path_by_file = '%s/%s/%s' % (current_path, next_path_by_file, fname)

                # to check whenever new path is not created yet
                # '/path/to/env-welut/welut/welut_demo/media/ebooks/2017/11/nama-file/'
                next_path_dirname = os.path.split(next_path_by_file)[0]
                if not os.path.exists(next_path_dirname):
                    os.makedirs(next_path_dirname)

                # start path before moving the files
                source = os.path.join(prev_path, fname)
                destination = next_path_by_file
                if os.path.isfile(source) and self.get_file_extension(source) == WELUT_IMAGES_EXTENSION:
                    shutil.move(source, destination)
                    list_filenames.append(fname)

            # removing all file uploaded with specific extensions
            # this condition is process as last condition
            for ext in WELUT_REMOVED_EXTENSIONS:
                if os.path.isfile(epub_file_full_path):
                    if ext == '.epub' or ext == '.mobi':
                        os.remove(epub_file_full_path)
                    elif ext == '.pdf':
                        os.remove(pdf_file_full_path)

            return list_filenames

        except Exception as e:
            raise e



# -*- coding: utf-8 -*-

import os
import ast

from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from .settings import WELUT_EXTENSIONS
from .converter import FileConverter


def conv_validators(value):
    ext = '%s' % os.path.splitext(value.name)[-1]
    if ext.lower() not in WELUT_EXTENSIONS:
        message = _('Invalid format, please choose: %(ext)s') % {'ext': WELUT_EXTENSIONS}
        raise ValidationError(message)


class EbookConverter(models.Model):
    path = models.FileField(_('Ebook file'), upload_to='ebooks/%Y/%m/%d',
                            validators=[conv_validators])
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    def __str__(self):
        return _('file: %(file)s') % {'file': self.id}

    def get_files(self):
        """ return path + filename for all files """
        if self.path is not None:
            try:
                imgs_path_name = os.path.splitext(self.path.name)[0]
                imgs_path_full = os.path.splitext(self.path.path)[0]
                listdir = [path for path in os.listdir(imgs_path_full)]
                return list(os.path.join(imgs_path_name, fname) for fname in listdir)
            except Exception:
                return list()
        return list()
    get_files.allow_tags = True
    get_files.short_description = _('List of files')

    def save(self, *args, **kwargs):
        super(EbookConverter, self).save(*args, **kwargs)
        fc = FileConverter()
        if not fc.convert(self.path):
            raise Exception(_('Cannot convert the file!'))
        return True

    class Meta:
        verbose_name = _('Ebook Converter')
        verbose_name_plural = _('Ebooks Converter')
        ordering = ['-created']



# -*- coding: utf-8 -*-

from os import listdir, rename
from os.path import isfile, join
import subprocess
import os
import glob

# return name of file to be kept after conversion.
# we are just changing the extension. azw3 here.


def get_final_filename(f):
    f = f.split(".")
    filename = ".".join(f[0:-1])
    processed_file_name = filename + ".pdf"
    return processed_file_name

# return file extension. pdf or epub or mobi


def get_file_extension(f):
    return f.split(".")[-1]

# list of extensions that needs to be ignored.
ignored_extensions = ["pdf"]

# here all the downloaded files are kept
mypath = "/home/faketaxi/DANIAS/Project_django/skripsi/coba2/bashrc/a/"  # "/home/user/Downloads/ebooks/"


raw_files = [f for f in listdir(mypath) if isfile(join(mypath, f))]
converted_files = [f for f in listdir(mypath) if isfile(join(mypath, f))]

for f in raw_files:
    final_file_name = get_final_filename(f)
    extension = get_file_extension(f)
    if final_file_name not in converted_files and extension not in ignored_extensions:
        print("Converting : " + f)
        try:
            # convert epub / mobi to pdf
            subprocess.call(["ebook-convert", mypath + f, mypath + final_file_name])
            subprocess.call(['rm', '-r', mypath + f])  # remove epub / mobi
            # convert pdf to png
            subprocess.call(['pdftocairo', '-png', mypath + final_file_name, mypath])
            subprocess.call(['rm', '-r', mypath + final_file_name])  # remove pdf
            s = rename(mypath + f)
            print(s)
        except Exception as e:
            print(e)
    else:
        print("Already exists : " + final_file_name)


#######
#######


def file_converter(_file):
    # _file.path =
    # /home/agaust/ENV/env-welut/welut/welut_demo/media/ebooks/2017/11/13/pg55939_m9m4VI4.epub
    full_epub_path = _file.path
    splited_path = os.path.split(full_epub_path)
    current_path, origin_file = splited_path[0], splited_path[-1]
    full_pdf_path = os.path.join(current_path, get_final_filename(full_epub_path, '.pdf'))

    try:
        subprocess.call(['ebook-convert', full_epub_path, full_pdf_path])
        #subprocess.call(['rm', '-r', full_epub_path])
        subprocess.call(['pdftocairo', '-png', full_pdf_path, current_path])

        # move all files from: 13-01.png to 13/filename-01.png
        # eg: '/home/agaust/ENV/env-welut/welut/welut_demo/media/ebooks/2017/11'

        # ('/home/agaust/ENV/env-welut/welut/welut_demo/media/ebooks/2017/11', '13')
        splited_path = os.path.split(current_path)

        # '/home/agaust/ENV/env-welut/welut/welut_demo/media/ebooks/2017/11'
        path_before = splited_path[0]

        # egg: '13'
        path_next = splited_path[-1]

        for fname in os.listdir(path_before):
            # fname.replace(get_file_extension(fname), '.png')  # 'foo.epub' => 'foo.png'
            new_fname = fname.replace(get_file_extension(fname), '.png')
            dir_next = os.path.basename(os.path.splitext(full_epub_path)[0])  # 'pg55939'
            file_next = '%s/%s/%s' % (current_path, dir_next, new_fname)
            new_dir = os.path.split(file_next)[0]

            if not os.path.exists(new_dir):
                os.makedirs(new_dir)

            file_before = os.path.join(path_before, fname)

            if os.path.isfile(file_before) and get_file_extension(file_before) == '.png':
                shutil.move(file_before, file_next)

        # removing all file uploaded with specific extensions
        for ext in WELUT_REMOVED_EXTENSIONS:
            if os.path.isfile(full_epub_path):
                if ext == '.epub' or ext == '.mobi':
                    os.remove(full_epub_path)
                elif ext == '.pdf':
                    os.remove(full_pdf_path)
        return True
    except Exception as e:
        raise e



# -*- coding: utf-8 -*-

from django.conf import settings

WELUT_EXTENSIONS = getattr(settings, 'WELUT_EXTENSIONS', ['.pdf', '.epub', '.mobi'])
WELUT_REMOVED_EXTENSIONS = getattr(settings, 'WELUT_REMOVED_EXTENSIONS', ['.pdf', '.epub', '.mobi'])
WELUT_IMAGES_EXTENSION = getattr(settings, 'WELUT_IMAGE_EXTENSION', '.png')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

from .models import EbookConverter


class EbookConverterAdmin(admin.ModelAdmin):
    list_display = ['__str__', 'path', 'created', 'modified']
    readonly_fields = ['get_files']


admin.site.register(EbookConverter, EbookConverterAdmin)
